SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CardTest methodsFor: 'expiration-tests' stamp: 'JuanEdi 6/8/2013 11:25'!
testShouldBeExpiredIfCurrentMonthIsAfterExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 3 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth).! !

!CardTest methodsFor: 'expiration-tests' stamp: 'JuanEdi 6/8/2013 11:26'!
testShouldNotBeExpiredIfCurrentMonthIsBeforeExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 1 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth) not.! !

!CardTest methodsFor: 'expiration-tests' stamp: 'JuanEdi 6/8/2013 11:26'!
testShouldNotBeExpiredIfCurrentMonthIsExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 2 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth) not.! !

TestCase subclass: #CashierTest
	instanceVariableNames: 'catalog calendar expiredCard validCard paymentTerminalStub cashier transactionCount lastCardUsed lastAmountProcessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CashierTest methodsFor: 'running' stamp: 'pb 6/9/2013 11:54'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.
	
	calendar := FixedCalendar setIn: (Date newDay: 1 month: 2 year: 2000 ).
	
	expiredCard := Card ownedBy: 'Marcelo' withNumber: 123 expiring: (Month month: 1 year: 2000).
	validCard := Card ownedBy: 'Marcelo' withNumber: 456 expiring: (Month month: 3 year: 2000).
	
	paymentTerminalStub := PluggableStubPaymentTerminal new.
	transactionCount := 0.
	
	cashier := Cashier usingCatalog: catalog paymentTerminal: paymentTerminalStub andCalendar: calendar.
! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:54'!
testShouldDebitTheCartAmountInTheTerminalWhenCheckingOutACart
	| aCart expectedAmount |
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	
	paymentTerminalStub onDebitDo: [ :anAmount :aCard |
		transactionCount := transactionCount + 1.
		lastCardUsed  := aCard.
		lastAmountProcessed := anAmount. ].
	
	cashier checkout: aCart usingCard: validCard.
	
	expectedAmount := (2 * (catalog at: 'Harry Potter')) +  (3 * catalog at: 'Dracula').
	
	self assert: lastCardUsed = validCard.
	self assert: lastAmountProcessed = expectedAmount.! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:54'!
testShouldFailIfTryingToCheckoutEmptyCart
	
	| initialTransactionCount registeredSalesSize |
	
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1. ].
	registeredSalesSize := cashier salesBook size. 
	
	self should: [ cashier checkout: (ShoppingCart newWithCatalog: catalog) usingCard: '12345' ]
		  raise: Error
		  withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier emptyCartCheckoutMessage. 
			self assert: transactionCount = initialTransactionCount.
			self assert: cashier salesBook size = registeredSalesSize.
		  ]! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:54'!
testShouldFailIfTryingToCheckoutWithExpiredCreditCard
	
	| aCart initialTransactionCount registeredSalesSize |
	
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1. ].
	registeredSalesSize := cashier salesBook size. 
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
		
	self should: [ cashier checkout: aCart usingCard: expiredCard ]
		  raise: Error
		  withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier expiredCardCheckoutMessage. 
			self assert: transactionCount = initialTransactionCount.
			self assert: cashier salesBook size = registeredSalesSize.
		  ]! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/5/2013 19:49'!
testShouldFailIfTryingToUseCardWithNoCredit
	
	| aCashier aPaymentTerminal aCart registeredSalesSize |
	
	aPaymentTerminal := PluggableStubPaymentTerminal new.
	aPaymentTerminal onDebitDo: [ :anAmount :aCard | CardWithNoCreditError signal ].
	
	aCashier := Cashier usingCatalog: catalog paymentTerminal: aPaymentTerminal andCalendar: calendar.
	
	registeredSalesSize := aCashier salesBook size. 
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	self should: [ aCashier checkout: aCart usingCard: validCard ]
		  raise: CardWithNoCreditError
		  withExceptionDo: [ :anException | 
			self assert: aCashier salesBook size = registeredSalesSize.
		  ]! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:56'!
testShouldPrintTicketWhenCheckingOutAValidCart
	
	| aCart ticket expectedAmount |
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	
	ticket := cashier checkout: aCart usingCard: validCard.
	
	expectedAmount := (2 * (catalog at: 'Harry Potter')) +  (3 * catalog at: 'Dracula').
	
	self assert: ticket amount = expectedAmount.
	! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:54'!
testShouldProcessAndRegisterTransactionWhenCheckingOutValidCart
	
	| aCart initialTransactionCount salesCount |
	
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1. ].
	salesCount := cashier salesBook size. 
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	
	cashier checkout: aCart usingCard: validCard.
	
	self assert: transactionCount = (initialTransactionCount + 1).
	self assert: cashier salesBook size = (salesCount + 1).! !

!CashierTest methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:54'!
testShouldRegisterTheSaleWithTheRightAmountWhenCheckingOutACart
	| aCart expectedAmount lastSale |
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1. ].
	
	cashier checkout: aCart usingCard: validCard.
	
	expectedAmount := (2 * (catalog at: 'Harry Potter')) +  (3 * catalog at: 'Dracula').
	
	lastSale := cashier salesBook last.
	
	self assert: lastSale totalAmount = expectedAmount.
	! !

TestCase subclass: #RestInterfaceTest
	instanceVariableNames: 'cartList userRegistry catalog restInterface bookRegistry clock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RestInterfaceTest methodsFor: 'running' stamp: 'FW 6/8/2013 19:08'!
setUp
	
	cartList := Dictionary new.
	userRegistry := Dictionary new.
	bookRegistry := Dictionary new.
	catalog := Dictionary new.
	
	userRegistry at: 1 put: '1111'.
	bookRegistry at: 'ISBN1' put: 'Harry Potter'.
	catalog at: 'Harry Potter' put: 10.
	
	clock := FixedClock new.
	
	restInterface := RestInterface newUsingCartList: cartList
										userRegistry: userRegistry
										bookRegistry: bookRegistry
										catalog: catalog
										andClock: clock.! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'JuanEdi 6/5/2013 21:25'!
testReturnCartIdAfterCreation

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self assert: cartList size = 1.
	self assert: (cartList keys includes: cartId).
	self assert: (cartList at: cartId) isEmpty.! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'JuanEdi 6/8/2013 11:36'!
testShouldAddElementsToCart
	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := cartList at: cartId.
	
	restInterface addToCart: cartId isbn: 'ISBN1' copies: 2.
 
	self assert: cart itemCount = 2.
	self assert: (cart occurrencesOf: 'Harry Potter') = 2.	! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'JuanEdi 6/5/2013 21:25'!
testShouldFailIfCreatingCartForNonexistentUser

	self	should: [ restInterface newCartForUserWithId: 1 andPassword: '1234' ]
			raise:  AuthenticationFailed
			withExceptionDo: [ :anError |
				self assert: anError messageText = RestInterface invalidAuthenticationData. 
				self assert: cartList isEmpty.
			]! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'JuanEdi 6/5/2013 21:25'!
testShouldFailIfCreatingCartWithInvalidUserPassword

	self	should: [ restInterface newCartForUserWithId: 1 andPassword: '1234' ]
			raise:  AuthenticationFailed
			withExceptionDo: [ :anError |
				self assert: anError messageText = RestInterface invalidAuthenticationData. 
				self assert: cartList isEmpty.
			]! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'JuanEdi 6/8/2013 12:18'!
testShouldFailIfTryingToAddInexistentBook

	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := cartList at: cartId.
	
	self should: [ restInterface addToCart: cartId isbn: 'ISBN40' copies: 2. ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownBookMessage.
			self assert: cartList size = 1.
			self assert: cart isEmpty.
		].! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'pb 6/9/2013 12:05'!
testShouldFailIfTryingToAddItemsToACartAfter30MinutesOfInactivity

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	clock add: (Duration minutes: 41).
	
	self should: [ restInterface addToCart: cartId isbn: 'ISBN1' copies: 2. ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface inactivityTimeoutMessage.
			self assert: cartList size = 0
		].! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'JuanEdi 6/8/2013 12:14'!
testShouldFailIfTryingToAddItemsToInexistentCart

	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := cartList at: cartId.
	
	self should: [ restInterface addToCart: (cartId + 1) isbn: 'ISBN1' copies: 2. ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownCartMessage.
			self assert: cartList size = 1.
			self assert: cart isEmpty.
		].! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'JuanEdi 6/5/2013 21:33'!
testShouldGenerateNewIdsForEveryCart

	| firstCartId secondCartId |
	
	firstCartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	secondCartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self assert: cartList size = 2.
	self assert: (firstCartId = secondCartId) not.! !

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: 'catalog calendar expiredCard validCard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: 'running' stamp: 'JuanEdi 6/2/2013 01:06'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:54'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: catalog) isEmpty.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOccurrences: 2.
	self assert: (cart occurrencesOf: 'Dracula') = 2.
	self assert: (cart itemCount) = 2.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldFailIfTryingToAddANegativeAmountOfProducts

	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter' withOccurrences: 2.
	self should: [ cart add: 'Harry Potter' withOccurrences: -1 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart invalidAmountOfProductsMessage ].
	self assert: (cart occurrencesOf: 'Harry Potter') = 2.
	self assert: (cart itemCount) = 2.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	self should: [ cart add: 'Cien aos de soledad' withOccurrences: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOccurrences: 2 .
	cart add: 'Dracula' withOccurrences: 3.
	self assert: (cart occurrencesOf: 'Dracula') = 5.
	self assert: (cart itemCount) = 5.! !

Error subclass: #AuthenticationFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Error subclass: #CardWithNoCreditError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Object subclass: #Calendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Calendar subclass: #FixedCalendar
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!FixedCalendar commentStamp: '<historical>' prior: 0!
A FixedCalendar is useful when there is a need to fix the date used as the current date during a test.

Instance Variables
	date:		<Date>

date
	- xxxxx
!

!FixedCalendar class methodsFor: 'setup' stamp: 'JuanEdi 6/2/2013 00:38'!
setIn: aDate

	^ self new setIn: aDate.! !

!FixedCalendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:37'!
currentMonth

	^ date month.! !

!FixedCalendar methodsFor: 'initialize-release' stamp: 'JuanEdi 6/2/2013 00:44'!
setIn: aDate

	date := aDate.! !

Calendar subclass: #RealTimeCalendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RealTimeCalendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:55'!
currentMonth

	^ Date today month.! !

Object subclass: #Card
	instanceVariableNames: 'number expirationMonth ownerName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Card class methodsFor: 'instance-creation' stamp: 'JuanEdi 6/8/2013 11:17'!
ownedBy: ownerName withNumber: aCardNumber expiring: aMonth 
	
	^ self new initializeWithOwner: ownerName number: aCardNumber expiringOn: aMonth.! !

!Card methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:35'!
expirationMonth
	
	^ expirationMonth! !

!Card methodsFor: 'initialize-release' stamp: 'JuanEdi 6/8/2013 11:23'!
initializeWithOwner: aName number: aCardNumber expiringOn: aMonth
	
	ownerName := aName.
	number := aCardNumber.
	expirationMonth := aMonth.! !

!Card methodsFor: 'accessing' stamp: 'JuanEdi 6/8/2013 11:21'!
isExpiredWithCurrentMonthBeing: currentMonth 
	
	^ expirationMonth < currentMonth.! !

Object subclass: #Cashier
	instanceVariableNames: 'paymentTerminal catalog calendar salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:33'!
emptyCartCheckoutMessage
	
	^ 'Cannot checkout empty carts.'! !

!Cashier class methodsFor: 'error-messages' stamp: 'JuanEdi 6/2/2013 00:23'!
expiredCardCheckoutMessage
	
	^ 'Cannot checkout a cart with an expired card'! !

!Cashier class methodsFor: 'instance-creation' stamp: 'JuanEdi 6/2/2013 00:31'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	^ self new usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar.! !

!Cashier methodsFor: 'checkout' stamp: 'pb 6/9/2013 11:57'!
checkout: aCart usingCard: aCard
	| totalAmount sale |
	self validateCartIsNotEmpty: aCart.
	self validateCardIsNotExpired: aCard.
	
	totalAmount := 0.
	aCart do: [ :anItem | totalAmount := totalAmount + (catalog at: anItem) ].
	paymentTerminal debit: totalAmount from: aCard.
	
	sale := Sale newWithAmount: totalAmount.
	salesBook add: sale.
	
	^Ticket newWithAmount: totalAmount.! !

!Cashier methodsFor: 'sales' stamp: 'pb 6/2/2013 16:28'!
salesBook
	^ salesBook.! !

!Cashier methodsFor: 'initialize-release' stamp: 'pb 6/2/2013 16:24'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	catalog := aCatalog.
	paymentTerminal := aPaymentTerminal.
	calendar := aCalendar.
	salesBook := OrderedCollection new.! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/8/2013 11:25'!
validateCardIsNotExpired: aCard 
	(aCard isExpiredWithCurrentMonthBeing: calendar currentMonth)
		ifTrue: [self error: Cashier expiredCardCheckoutMessage]! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/2/2013 01:00'!
validateCartIsNotEmpty: aShoppingCart 
	aShoppingCart isEmpty ifTrue: [self error: Cashier emptyCartCheckoutMessage]! !

Object subclass: #FixedClock
	instanceVariableNames: 'duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!FixedClock methodsFor: 'adding' stamp: 'FW 6/8/2013 19:08'!
add: aTimeDuration

	duration := duration + aTimeDuration! !

!FixedClock methodsFor: 'converting' stamp: 'FW 6/8/2013 19:09'!
asMinutes
	
	^ duration asMinutes.! !

!FixedClock methodsFor: 'initialize-release' stamp: 'FW 6/8/2013 19:09'!
initialize
	duration := Duration new! !

Object subclass: #PluggableStubPaymentTerminal
	instanceVariableNames: 'onDebitClosure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PluggableStubPaymentTerminal methodsFor: 'debit-actions' stamp: 'FranciscoWesner 6/4/2013 17:57'!
debit: anAmount from: aCard 
	
	onDebitClosure value: anAmount value: aCard.! !

!PluggableStubPaymentTerminal methodsFor: 'initialize-release' stamp: 'pb 6/9/2013 11:54'!
initialize

	onDebitClosure := [:x :y | nil ].! !

!PluggableStubPaymentTerminal methodsFor: 'debit-actions' stamp: 'JuanEdi 6/3/2013 21:04'!
onDebitDo: aBlock

	onDebitClosure := aBlock.! !

Object subclass: #RestInterface
	instanceVariableNames: 'activeCartList lastTImeCartUsedList userRegistry catalog lastId bookRegistry clock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RestInterface class methodsFor: 'error-messages' stamp: 'FW 6/8/2013 17:52'!
inactivityTimeoutMessage
	^ 'Trying to operate with a cart after 30 minutes of inactivity'! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/5/2013 20:46'!
invalidAuthenticationData
	^ 'Invalid Authentication data'! !

!RestInterface class methodsFor: 'instance-creation' stamp: 'FW 6/8/2013 17:51'!
newUsingCartList: aCartList userRegistry: aUserRegistry bookRegistry: aBookRegistry catalog: aCatalog andClock: aClock.
	
	^ self new withActiveCartList: aCartList userRegistry: aUserRegistry bookRegistry: aBookRegistry catalog: aCatalog andClock: aClock.! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/8/2013 12:18'!
unknownBookMessage
	^ 'The book does not exist.'! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/8/2013 11:57'!
unknownCartMessage
	^ 'Trying to operate with unknown cart'! !

!RestInterface methodsFor: 'adding' stamp: 'FW 6/8/2013 18:42'!
addToCart: aCartId isbn: anIsbn copies: numberOfCopies
	| aCart aBook |
	self validateCartExists: aCartId.
	self validateBookExists: anIsbn.
	self validateActiveCart: aCartId.
	
	aCart := activeCartList at: aCartId.
	aBook := bookRegistry at: anIsbn.
	
	aCart add: aBook withOccurrences: numberOfCopies.! !

!RestInterface methodsFor: 'instance-creation' stamp: 'FW 6/8/2013 18:16'!
newCartForUserWithId: aUserId andPassword: aPassword 

	| cartId |

	self validateUserExists: aUserId.
	self validateUser: aUserId hasPassword: aPassword.
	
	cartId := self newCartId.
	activeCartList at: cartId put: (ShoppingCart newWithCatalog: catalog).
	lastTImeCartUsedList at: cartId put: (clock asMinutes).
	
	^ cartId.! !

!RestInterface methodsFor: 'instance-creation' stamp: 'JuanEdi 6/5/2013 21:51'!
newCartId
	
	| newId |
	newId := lastId + 1.
	lastId := newId.
		
	^ newId.! !

!RestInterface methodsFor: 'validating' stamp: 'FW 6/8/2013 19:14'!
validateActiveCart: aCartId

	((clock asMinutes) - (lastTImeCartUsedList at: aCartId) > 30)
		ifTrue: [ lastTImeCartUsedList removeKey: aCartId.
				 activeCartList removeKey: aCartId.
				 self error: RestInterface inactivityTimeoutMessage. ].
		
	lastTImeCartUsedList at: aCartId put: (clock asMinutes).! !

!RestInterface methodsFor: 'validating' stamp: 'JuanEdi 6/8/2013 12:20'!
validateBookExists: anIsbn

	(bookRegistry includesKey: anIsbn)
		ifFalse: [ self error: RestInterface unknownBookMessage ]! !

!RestInterface methodsFor: 'validating' stamp: 'JuanEdi 6/8/2013 12:15'!
validateCartExists: aCartId

	(activeCartList includesKey: aCartId)
		ifFalse: [ self error: RestInterface unknownCartMessage ]! !

!RestInterface methodsFor: 'validating' stamp: 'JuanEdi 6/5/2013 21:01'!
validateUser: aUserId hasPassword: aPassword
	((userRegistry at: aUserId) = aPassword) ifFalse:
		[AuthenticationFailed signal: RestInterface invalidAuthenticationData]! !

!RestInterface methodsFor: 'validating' stamp: 'JuanEdi 6/5/2013 21:01'!
validateUserExists: aUserId
	(userRegistry keys includes: aUserId) ifFalse:
		[AuthenticationFailed signal: RestInterface invalidAuthenticationData]! !

!RestInterface methodsFor: 'instance-creation' stamp: 'FW 6/8/2013 18:19'!
withActiveCartList: aCartList userRegistry: aUserRegistry bookRegistry: aBookRegistry catalog: aCatalog andClock: aClock
	
	activeCartList := aCartList.
	lastTImeCartUsedList := Dictionary new.
	userRegistry := aUserRegistry.
	bookRegistry := aBookRegistry.
	catalog := aCatalog.
	clock := aClock.
	lastId := 0.! !

Object subclass: #Sale
	instanceVariableNames: 'totalAmount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'instance-creation' stamp: 'pb 6/2/2013 16:37'!
newWithAmount: anAmount 
	^self new intializeWithAmount: anAmount. ! !

!Sale methodsFor: 'initialize-release' stamp: 'pb 6/2/2013 16:37'!
intializeWithAmount: anAmount 
	totalAmount := anAmount.
	! !

!Sale methodsFor: 'accessing' stamp: 'pb 6/2/2013 16:38'!
totalAmount
	^ totalAmount! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:23'!
invalidAmountOfProductsMessage
	
	^ 'Amount of products to add should be greater than zero'.! !

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/8/2013 12:23'!
add: aProduct 
	self add: aProduct withOccurrences: 1.! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/8/2013 12:23'!
add: aProduct withOccurrences: aNumber 
	
	self validatePossitiveAmountOfProducts: aNumber.
	self validateCatalogContainsProduct: aProduct.
		
	products add: aProduct withOccurrences: aNumber.! !

!ShoppingCart methodsFor: 'enumerating' stamp: 'JuanEdi 6/1/2013 22:54'!
do: aBlock
	"Itera sobre cada ocurrencia individual de cada producto del carro."

	products do: aBlock! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 6/1/2013 22:24'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Bag new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'accessing' stamp: 'JuanEdi 6/1/2013 22:31'!
itemCount
	"Retorna la cantidad de items en total, contando todas las unidades de cada item agregado."
	^ products size.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 6/1/2013 22:55'!
occurrencesOf: aProduct 
	
	^ products occurrencesOf: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 6/1/2013 22:34'!
validateCatalogContainsProduct: aProduct

	(catalog keys includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:22'!
validatePossitiveAmountOfProducts: anAmount

	(anAmount > 0) ifFalse: [ self error: ShoppingCart invalidAmountOfProductsMessage]! !

Object subclass: #Ticket
	instanceVariableNames: 'amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Ticket class methodsFor: 'instance-creation' stamp: 'pb 6/9/2013 11:59'!
newWithAmount: anAmount 
	^self new initializeWith: anAmount.! !

!Ticket methodsFor: 'accessing' stamp: 'pb 6/9/2013 11:59'!
amount
	^ amount! !

!Ticket methodsFor: 'initialize-release' stamp: 'pb 6/9/2013 11:58'!
initializeWith: anAmount 
	amount := anAmount.! !
